const parser = require('@typescript-eslint/parser');
const parseForESLint = parser.parseForESLint;
const babelVisitorKeys = require('@babel/types').VISITOR_KEYS;

// Monkey-patch: override typescript-eslint-parser's visitorKeys
// issue: https://github.com/eslint/typescript-eslint-parser/issues/514
parser.parseForESLint = function() {
  const result = parseForESLint.apply(this, arguments);
  result.visitorKeys = babelVisitorKeys;
  return result;
};

module.exports = {
  parser: '@typescript-eslint/parser',
  plugins: [
    '@typescript-eslint',
  ],
  rules: {
    'no-undef': 'off',
    'no-unused-vars': 'off',
    'no-useless-constructor': 'off',
    'no-dupe-class-members': 'off',
    'no-use-before-define': 'off',
    '@typescript-eslint/adjacent-overload-signatures': 'error',
    '@typescript-eslint/array-type': ['error', {
      'default': 'array',
      'readonly': 'array',
    }],
    '@typescript-eslint/ban-types': ['error', {
      'types': {
        'Object': 'Use {} instead',
        'String': {
          'message': 'Use string instead',
          'fixWith': 'string'
        }
      }
    }],
    '@typescript-eslint/class-name-casing': 'error',
    '@typescript-eslint/member-delimiter-style': 'error',
    '@typescript-eslint/no-array-constructor': 'error',
    '@typescript-eslint/no-extraneous-class': 'error',
    '@typescript-eslint/no-inferrable-types': 'error',
    '@typescript-eslint/no-misused-new': 'error',
    '@typescript-eslint/no-namespace': 'error',
    '@typescript-eslint/no-non-null-assertion': 'warn',
    '@typescript-eslint/consistent-type-assertions': 'error',
    '@typescript-eslint/consistent-type-definitions': 'error',
    '@typescript-eslint/func-call-spacing': 'error',
    '@typescript-eslint/no-useless-constructor': 'error',
    '@typescript-eslint/prefer-for-of': 'off',

    '@typescript-eslint/explicit-member-accessibility': ['error', {
      'accessibility': 'no-public',
      'overrides': {
        'parameterProperties': 'off'
      }
    }],

    // rules have not been prepared
    // https://github.com/eslint/typescript-eslint-parser/issues/457
    // '@typescript-eslint/no-unused-vars': ['error', {
    //   'argsIgnorePattern': '^_'
    // }],
    '@typescript-eslint/no-use-before-define': ['error', {
      'functions': false,
      'classes': false,
      'typedefs': false
    }],
    '@typescript-eslint/no-var-requires': 'error',

    // rules have not been prepared
    // https://github.com/typescript-eslint/typescript-eslint/issues/130
    // '@typescript-eslint/restrict-plus-operands': 'error',
    '@typescript-eslint/type-annotation-spacing': 'error'
  }
};
